package abacomayadecimale;

/**
 * Classe per l'implementazione dell'abaco maya in forma decimale e
 * permette l'esecuzione di addizioni, sottrazioni, moltiplicazioni e divisioni.
 * Necessita delle classi Cell, SCell e DCell per il corretto funzionamento.
 * @see Cell
 * @see SCell
 * @see DCell
 */
public class Abaco {

    /**
     * Costante per determinare l'operazione di addizione.
     */
    public final static int SUM = 0;
    
    /**
     * Costante per determinare l'operazione di sottrazione.
     */
    public final static int SUB = 1;
    
    /**
     * Costante per determinare l'operazione di moltiplicazione.
     */
    public final static int MUL = 2;
    
    /**
     * Costante per determinare l'operazione di divisione.
     */
    public final static int DIV = 3;
    
    /**
     * Matrice di SCell formata da 4 righe e 4 colonne necessaria allo svolgimento
     * dell'addizione, sottrazione e divisione.
     */
    protected SCell SMatrix [][];
    
    /**
     * Matrice di DCell formata da 4 righe e 4 colonne necessaria allo svolgimento
     * della moltiplicazione.
     */
    protected DCell DMatrix [][];
    
    /**
     * Vettore di SCell formato da 4 elementi situato nella parte superiore dell'abaco.
     * Viene utilizzato:
     * <ul>
     * <li> nella moltiplicazione per il valore del moltiplicando
     * <li> nella divisione per il valore del risultato dell'operazione
     * </ul>
     */
    protected SCell up[];
        
    /**
     * Vettore di SCell formato da 4 elementi situato a destra dell'abaco.
     * Viene utilizzato:
     * <ul>
     * <li> nella moltiplicazione per il valore del moltiplicatore
     * <li> nella divisione per il valore del divisore
     * </ul>
     */
    protected SCell right[];
    
    /**
     * Vettore di SCell formato da 4 elementi situato nella parte inferiore dell'abaco.
     * Viene utilizzato:
     * <ul>
     * <li> nell' addizione per le prime 4 cifre del risultato dell'operazione
     * <li> nella sottrazione per le prime 4 cifre del risultato dell'operazione
     * <li> nella moltiplicazione per le prime 4 cifre del risultato dell'operazione
     * <li> nella divisione per il valore del dividendo
     * </ul>
     */
    protected SCell down[];
    
    /**
     * Vettore di SCell formato da 4 elementi situato a sinistra dell'abaco.
     * Viene utilizzato:
     * <ul>
     * <li> nell'addizione come completamento del risultato dell'operazione in caso sia necessaria un unlteriore cifra.
     * <li> nella moltiplicazione come completamento del risultato dell'operazione in caso non siano bastate le celle di <code>down</code>.
     * <li> nella divisione per il valore del dividendo
     * </ul>
     */
    protected SCell left[];
    
    /**
     * Stringa che contiene la descrizione dello step corrente dell'operazione in corso.
     */
    protected String str = "";    
    
    /**
     * Costruttore dell'oggetto Abaco.
     * Vengono inizializzati tutti gli oggetti dell'abaco per il corretto svolgimento
     * dei metodi.
     */
    Abaco(){
                                      
        SMatrix = new SCell[4][4];
        DMatrix = new DCell[4][4];
        up = new SCell[4];
        right = new SCell[4];
        down = new SCell[4];
        left = new SCell[4];
        
        //Inizializza gli operandi e il risultato
        for(int i = 0; i<4; i++){
            up[i] = new SCell(SCell.UP, -1, i, false, false, true, false, false, false, false, true);
            right[i] = new SCell(SCell.RIGHT, i, -1, false, false, true, true, false, false, false, false);
            down[i] = new SCell(SCell.DOWN, -1, i, false, false, false, true, true, true, true, false);
            left[i] = new SCell(SCell.LEFT, i, -1, false, false, false, false, i==3, false, true, false);
        }
        
        //Inizializza la matrice di SCell
        for(int i = 0; i<4; i++){
            for(int j = 0; j<4; j++){
                SMatrix[i][j] = new SCell(SCell.ABACO, i, j, i<3, i<2, false, false, i>2, i==0, false, i+j>2);
            }
        }
        
        //Inizializza la matrice di DCell
        int number = 8;
        for(int i = 0; i<4; i++){
            for(int j = 0; j<4; j++){                
                if(j == 0){
                    if(i == 3)
                        DMatrix[i][j] = new DCell(i, j, number--, number--);
                    else DMatrix[i][j] = new DCell(i, j, number--, -1);
                
                } else if(i == 3){
                    DMatrix[i][j] = new DCell(i, j, -1, number--);
                
                } else { 
                    DMatrix[i][j] = new DCell(i, j);
                }
            }
        }
    }
    
    /**
     * Converte gli elementi della riga specificata da <code>row</code> contenuta
     * in <code>SMatrix</code> e ritorna un intero contenente il rispettivo valore.
     * @param row il numero di riga richiesto
     * @return un intero contenente il valore della riga specificata
     */
    public int getRow(int row){
        int x = 0;
        for(int i = 0; i<4; i++)
            x += SMatrix[row][i].getValue() * Math.pow(10, 3-i);
        
        return x;
    }

    /**
     * Converte gli elementi della colonna specificata da <code>col</code> contenuta
     * in <code>SMatrix</code> e ritorna un intero contenente il rispettivo valore.
     * @param col il numero di colonna richiesto
     * @return un intero contenente il valore della colonna specificata
     */
    public int getColumn(int col){
        int x = 0;
        for(int i = 0; i<4; i++)
            x += SMatrix[i][col].getValue() * Math.pow(10, 3-i);
        
        return x;
    }
    
    /**
     * Converte gli elementi dell'array <code>cells</code> e ritorna un intero contenente il rispettivo valore.
     * @param cells array di SCell di cui interessa conoscere il valore
     * @return un intero contenente il valore dell'array
     */
    public int getValueOf(SCell[] cells){
        int x = 0;
        for(int i = 0; i < 4; i++)
            x += cells[i].getValue()* Math.pow(10, 3-i);
        
        return x;
    }
    
    /**
     * Restituisce il valore del risultato.
     * In base all'operazione restituisce il valore del risultato.
     * @return un intero contenente il valore del risultato
     */
    public int getResult(){
        return this.getValueOf(left)*10000 + this.getValueOf(down);
    }

    /**
     * Ritorna un intero contenente il numero di celle non vuote nell'array <code>cells</code>.
     * @param cells array di SCell da controllare
     * @return un intero contenente il numero di celle non vuote.
     */
    public int getOccupiedCells(SCell[] cells){
        int x = 0;
        for(int i = 0; i<4; i++)
            if(cells[i].getValue() > 0 || cells[i].isCaracol())
                x++;
        
        return x;
    }
    
    /**
     * Toglie tutti gli oggetti in ogni cella della riga specificata 
     * da <code>row</code> contenuta in SMatrix.
     * @param row il numero di riga richiesto
     */
    public void resetRow(int row){
        for(int i = 0; i<4; i++)
            SMatrix[row][i].reset();
    }
    
    /**
     * Toglie tutti gli oggetti in ogni cella della colonna specificata 
     * da <code>col</code> contenuta in SMatrix.
     * @param col il numero di colonna richiesto
     */
    public void resetColumn(int col){
        for(int i = 0; i<4; i++)
            SMatrix[i][col].reset();
    }
    
    /**
     * Toglie tutti gli oggetti in ogni cella dell'array di SCell specificato.
     * @param cells array di SCell da svuotare
     */
    public void resetOperand(SCell[] cells){
        for(int i = 0; i<4; i++)
            cells[i].reset();
    }
    
    /**
     * Toglie tutti gli oggetti in ogni cella di <code>SMatrix</code>.
     */
    public void resetSMatrix(){
        for(int i = 0; i<4; i++)
            for(int j = 0; j<4; j++)
                SMatrix[i][j].reset();
    }
    
    /**
     * Toglie tutti gli oggetti in ogni cella di <code>DMatrix</code>.
     */
    public void resetDMatrix(){
        for(int i = 0; i<4; i++)
            for(int j = 0; j<4; j++)
                DMatrix[i][j].reset();
    }
    
    /**
     * Toglie tutti gli oggetti in ogni cella dell'intero abaco.
     * Comprende le seguenti variabili di classe:
     * <ul>
     * <li><code>up</code>
     * <li><code>right</code>
     * <li><code>down</code>
     * <li><code>left</code>
     * <li><code>SMatrix</code>
     * <li><code>DMatrix</code>
     * </ul>
     */
    public void reset(){        
        this.resetOperand(up);
        this.resetOperand(right);
        this.resetOperand(down);
        this.resetOperand(left);
        this.resetSMatrix();
        this.resetDMatrix();
    }

    /**
     * Imposta il valore di ogni cifra di <code>value</code> nelle rispettive celle
     * della riga specificata da <code>row</code> in <code>SMatrix</code>.
     * @param row il numero di riga richiesto
     * @param value il valore da inserire nella riga
     */
    public void setRow(int row, int value){
	this.resetRow(row);
        //Crea la stringa con il valore da parsare
        String val = Integer.toString(value);
        //Imposta i valori nelle celle
        int j = val.length()-1;
        for(int i = 3; i >= 0 && j >=0; i--, j--)
            SMatrix[row][i].setValue(val.charAt(j)-48);
    }
    
    /**
     * Imposta il valore di ogni cifra di <code>value</code> nelle rispettive celle
     * della colonna specificata da <code>col</code> in <code>SMatrix</code>.
     * @param col il numero di colonna richiesto
     * @param value il valore da inserire nella colonna
     */
    public void setColumn(int col, int value){
        this.resetColumn(col);
        //Crea la stringa con il valore da parsare
        String val = Integer.toString(value);
        //Imposta i valori nelle celle
        int j = val.length()-1;
        for(int i = 3; i >= 0 && j >=0; i--, j--)
            SMatrix[i][col].setValue(val.charAt(j)-48);
    }
    
    /**
     * Imposta il valore di ogni cifra di <code>value</code> nelle rispettive celle
     * dell'array <code>cells</code>.
     * @param cells le celle a cui impostare il valore
     * @param value il valore da inserire nelle celle
     */
    public void setOperand(SCell[] cells, int value){
        //Svuota le celle
        this.resetOperand(cells);
        //Crea la stringa da parsare
        String val = Integer.toString(value);
        //Imposta i valori nelle celle
        int j = val.length()-1;
        for(int i = 3; i >= 0 && j >=0; i--, j--)
            cells[i].setValue(val.charAt(j)-48);
    }
    
    /**
     * Metodo ausiliario per lo svolgimento degli step dell'addizione.
     * Questo metodo somma gli oggetti della cella 3|<code>col</code> e
     * della cella <code>down</code>[<code>col</code>], li converte e infine li inserisce
     * nelle celle opportune nell'array <code>down</code>.
     * Inoltre tale metodo provvede alla descrizione dello step in corso.
     * @param col il numero della colonna a cui applicare il metodo
     */
    private void sumCol(int col){
        
	int value = SMatrix[3][col].getValue() + down[col].getValue();
        int u = value%10;
        int d = value/10;
        
        down[col].setValue(u);

        int cell = 4-col;
        str = "Si inseriscono gli oggetti della cella 1|"+cell+" nella cella "+cell+" del risultato in basso.";
        
        if(d>0){
            if(col > 0){
                down[col-1].setValue(d);
                str = "Si convertono gli elementi della cella 1|"+cell+" e si inseriscono nella cella "+cell+" e "+(cell+1)+" del risultato in basso.";
                
            } else {
                left[3].setValue(d);
                str = "Si convertono gli elementi della cella 1|"+cell+" e si inseriscono nella cella "+cell+" del risultato in basso e nella cella 1 del risultato a destra.";
            }
        }
    }

    /**
     * Metodo ausiliario dell'addizione per inserire opportunamente la descrizione del risultato ottenuto.
     * L'inserimento della descrizione avviene soltando quando la somma dei tre
     * addendi è uguale al valore restituito dal metodo <code>getResult()</code>,
     * ovvero quando gli step successivi non sono necessari.
     * <br>Esempio: 20+15+4 = 39
     * <br>Per risolvere l'operazione sull'abaco sono necessari 4 step, questo metodo
     * verrà richiamato esattamente al 4° step poichè verrà raggiunto il risultato 
     * proprio in tale step.
     * @param addendOne
     * @param addendTwo
     * @param addendThree 
     */
    private void resultSum(int addendOne, int addendTwo, int addendThree){
        if(addendOne + addendTwo + addendThree == this.getResult())
            str += "\nIl risultato dell'addizione "+addendOne+"+"+addendTwo+"+"+addendThree+" è "+this.getResult()+".";
    }
    
    /**
     * Metodo da usare per lo svolgimento dell'addizione a step.
     * Questo metodo deve essere richiamato passando come parametro <code>n</code>
     * il numero dello step da eseguire. Gli steps per questa operazione vanno da 0 a 5.
     * Per il corretto svolgimento dell'operazione, ad ogni nuovo step da eseguire
     * (che sia superiore o inferiore), tale metodo deve essere richiamato per tutti
     * gli steps precedenti ad <code>n</code> partendo da 0 e proseguendo di 1
     * step alla volta.
     * Lo step 0 prepara l'abaco per lo svolgimento dell'operazione.
     * Lo step 1 raccoglie gli elementi degli operandi in SMatrix[3][*].
     * Gli steps da 2 a 5 provvedono alla somma colonna per colonna.
     * @param n lo step da eseguire
     */
    public void sum(int n){

        int addendOne = this.getRow(0);
        int addendTwo = this.getRow(1);
        int addendThree = this.getRow(2);
        
        switch(n){
            case 0:   
                str = "";
                this.resetRow(3);
                this.resetOperand(down);
                this.resetOperand(left);
                break;
                
            case 1:
                //Lunghezza massima degli operandi
                int length = 4 - Math.max(Math.max(this.getOccupiedCells(SMatrix[0]), this.getOccupiedCells(SMatrix[1])), this.getOccupiedCells(SMatrix[2]));
                for(int i = length; i<4; i++){
                    SMatrix[3][i].setFrijolito(SMatrix[0][i].getFrijolito() + SMatrix[1][i].getFrijolito() + SMatrix[2][i].getFrijolito());
                    SMatrix[3][i].setPalito(SMatrix[0][i].getPalito() + SMatrix[1][i].getPalito() + SMatrix[2][i].getPalito());  
                }
                str = "Gli oggetti degli addendi di uguale significatività vengono raccolti nelle rispettive celle della riga 1 dell'abaco.";
                break;
                
            case 2:
                this.sumCol(3);
                this.resultSum(addendOne, addendTwo, addendThree);
                break;
                
            case 3:
                this.sumCol(2);
                this.resultSum(addendOne, addendTwo, addendThree);
                break;
                
            case 4:
                this.sumCol(1);
                this.resultSum(addendOne, addendTwo, addendThree);
                break;
                
            case 5:
                this.sumCol(0);
                this.resultSum(addendOne, addendTwo, addendThree);
                break;
        }  
    }

    /**
     * Metodo ausiliario per lo svolgimento degli step della sottrazione.
     * Questo metodo sottrae gli oggetti di SMatrix[1][<code>col<code>] da
     * SMatrix[0][<code>col</code>] e li inserisce in down[<code>col</code>].
     * Nel caso in cui la sottrazione non è diretta, il metodo provvede al prelevamento
     * di 1 frijolito nella casella (o nelle caselle, in caso alcune di esse 
     * tranne l'ultima abbiano un caracol) in cui risiedono gli oggetti con più 
     * significatività, lo converte in 2 palito e li aggiunge alla cella del minuendo
     * in cui era necessario avere oggetti di valore superiore alla rispettiva cella
     * del sottraendo, infine si procede alla sottrazione.
     * Inoltre tale metodo provvede alla descrizione dello step in corso.
     * @param col il numero della colonna a cui applicare il metodo
     */
    private void subCol(int col){

        int cell = 4-col;
        
        //Sottrae il valore della casella del sottraendo alla rispettiva del minuendo
        int result = SMatrix[0][col].getValue() - SMatrix[1][col].getValue();
        
        if(result >= 0){
            //Caso in cui la sottrazione è diretta
            down[col].setValue(result);
            str = "Si estrae una copia della cella 3|"+cell+" del sottranedo dalla rispettiva cella del minuendo, il resto si inserisce nella cella "+cell+" del risultato in basso.";
       
        } else {
            //Caso in cui la sottrazione si può fare prelevando un unità dalla cella/e successiva/e
            //Aggiunge una decina alla cella 4|cell
            SMatrix[0][col].addValue(10);
            for(int i = col-1; i >= 0; i--){
                int next = SMatrix[0][i].getValue();
                //Controlla se la cella 4|cell-i ha almeno una unità da poter convertire in decina
                if(next>0){
                    //Sottrae l'unità da convertire
                    SMatrix[0][i].setValue(next-1);
                    break;
                
                //La cella 4|cell-i ha valore 0, quindi imposto il valore 9 dato che dovrà farsi prestare una decina dalla cella successiva
                } else SMatrix[0][i].setValue(9);
            }
            
            down[col].setValue(SMatrix[0][col].getValue() - SMatrix[1][col].getValue());
            str = "Si prende in prestito una decina dalla cella 4|"+(cell+1)+" e viene aggiunta alla cella 4|"+cell+" del minuendo.\n";
            str+= "Si estrae una copia della cella 3|"+cell+" del sottraendo dalla rispettiva cella del minuendo , il resto si inserisce nella cella "+cell+" del risultato in basso.";
        }  
    }
    
    
    /**
     * Metodo ausiliario della sottrazione per inserire opportunamente 
     * la descrizione del risultato ottenuto.
     * L'inserimento della descrizione avviene soltando quando la differenza è uguale 
     * al valore restituito dal metodo <code>getValueOf(down)</code>,
     * ovvero quando gli step successivi non sono necessari.
     * <br>Esempio: 20-5 = 15
     * <br>Per risolvere l'operazione sull'abaco sono necessari 3 step, questo metodo
     * verrà richiamato esattamente al 3° step poichè verrà raggiunto il risultato 
     * proprio in tale step.
     * @param addendOne
     * @param addendTwo
     * @param addendThree 
     */
    private void resultSub(int minuend, int subtrahend){
        if(minuend - subtrahend == this.getValueOf(down))
            str += "\nIl risultato della sottrazione "+minuend+"-"+subtrahend+" è "+this.getValueOf(down) +".";
    }
    
    /**
     * Metodo da usare per lo svolgimento della sottrazione a step.
     * Questo metodo deve essere richiamato passando come parametro <code>n</code>
     * il numero dello step da eseguire. Gli steps per questa operazione vanno da 0 a 4.
     * Per il corretto svolgimento dell'operazione, ad ogni nuovo step da eseguire
     * (che sia superiore o inferiore), tale metodo deve essere richiamato per tutti
     * gli steps precedenti ad <code>n</code> partendo da 0 e proseguendo di 1
     * step alla volta.
     * Lo step 0 prepara l'abaco per lo svolgimento dell'operazione.
     * Gli steps da 1 a 4 provvedono alla sottrazione colonna per colonna.
     * @param n lo step da eseguire
     */
    public void sub(int n){
                
        int minuend = this.getRow(0);
        int subtrahend = this.getRow(1);
        
        switch(n){
            case 0:
                str = "";
                this.resetOperand(down);
                break;
                
            case 1:
                this.subCol(3);
                this.resultSub(minuend, subtrahend);
                break;
            
            case 2:
                this.subCol(2);
                this.resultSub(minuend, subtrahend);
                break;
              
            case 3:
                this.subCol(1);
                this.resultSub(minuend, subtrahend);
                break;
               
            case 4:
                this.subCol(0);
                this.resultSub(minuend, subtrahend);
                break;
        }
    }
    
    /**
     * Metodo ausiliario per lo svolgimento degli step della moltiplicazione.
     * Il metodo è necessario per la somma in diagonale delle semicelle.
     * Questo metodo aggiunge ad <code>operand</code> i frijolito e i palito
     * che possiede <code>cell</code>.
     * @param operand la cella in cui aggiungere i frijolito e i palito
     * @param cell la cella da cui prelevare i frijolito e i palito
     * @param isDown boolean necessario per specificare se i frijolito e i palito da prelevare
     * si trovano nella semicella inferiore o superiore
     */
    private void addValue(SCell operand, DCell cell, boolean isDown){
        if(isDown){
            operand.setFrijolito(operand.getFrijolito() + cell.getDownFrijolito());
            operand.setPalito(operand.getPalito() + cell.getDownPalito());
        
        } else {
            operand.setFrijolito(operand.getFrijolito() + cell.getUpFrijolito());
            operand.setPalito(operand.getPalito() + cell.getUpPalito());  
        }    
    }
    
    /**
     * Converte gli oggetti di ogni cella per rendere il risultato della molitpilicazione
     * più leggibile.
     * Questo metodo viene chiamato dopo aver risolto i precedenti step della moltiplicazione
     * poichè alcune celle possono assumere valore superiore a 9 (ovvero 4 frijolito e 1 palito).
     */
    private void convertResult(){

        //Risultato di down
        int lenDown = 4 - this.getOccupiedCells(down);
        for(int i = 3; i>=lenDown; i--){
            int x = down[i].getValue();
            int u = x%10;   //Unità
            int d = x/10;   //Decine
            
            down[i].setValue(u);
            if(d > 0)
                if(i > 0)
                    down[i-1].addValue(x/10);
                else left[3].addValue(x/10);
            
        }
        
        //Risultato di left
        int lenLeft = 4 - this.getOccupiedCells(left);
        for(int i = 3; i>lenLeft; i--){
            int x = left[i].getValue();
            int u = x%10;   //Unità
            int d = x/10;   //Decine
             
            left[i].setValue(u);
            
            if(d > 0)
                left[i-1].addValue(d);
        }
    }
    
    /**
     * Metodo da usare per lo svolgimento della moltiplicazione a step.
     * Questo metodo deve essere richiamato passando come parametro <code>n</code>
     * il numero dello step da eseguire. Gli steps per questa operazione vanno da 0 a 10.
     * Per il corretto svolgimento dell'operazione, ad ogni nuovo step da eseguire
     * (che sia superiore o inferiore), tale metodo deve essere richiamato per tutti
     * gli steps precedenti ad <code>n</code> partendo da 0 e proseguendo di 1
     * step alla volta.
     * Lo step 0 prepara l'abaco per lo svolgimento dell'operazione.
     * Lo step 1 esegue le moltiplicazioni riga per colonna (ogni cella del moltiplicando
     * viene moltiplicata per ogni cella del moltiplicatore).
     * Gli steps da 2 a 9 provvedono a raccogliere gli oggetti di ogni diagonale.
     * Lo step 10 provvede alla sistemazione del risultato convertendo gli oggetti.
     * @param n lo step da eseguire
     */
    public void mul(int n){

        int multiplying = this.getValueOf(up);
        int multiplier = this.getValueOf(right);
        
        switch(n){
            case 0:
                str = "";
                this.resetDMatrix();
                this.resetOperand(down);
                this.resetOperand(left);
                break;
                
            case 1:
                //Determina il numero di cifre del moltiplicando
                int lengthUp = 4 - Integer.toString(this.getValueOf(up)).length();
                //Determina il numero di cifre del moltiplicatore
                int lengthRight = 4 - Integer.toString(this.getValueOf(right)).length();
                
                for(int i = 3; i >= lengthRight; i--)
                    for(int j = 3; j >= lengthUp; j--)
                        DMatrix[i][j].setValue(up[j].getValue()*right[i].getValue());
                
                str = "Si moltiplica ogni cella dell'operando in alto con ogni cella dell'operando a destra, inserendo i risultati nell'abaco.";
                break;
                
            case 2:
                addValue(down[3], DMatrix[3][3], true);
                str = "Si raccolgono gli oggetti della diagonale 1.";
                break;
                
            case 3:
                addValue(down[2], DMatrix[2][3], true);
                addValue(down[2], DMatrix[3][3], false);
                addValue(down[2], DMatrix[3][2], true);
                str = "Si raccolgono gli oggetti della diagonale 2.";
                break; 
                
            case 4:
                addValue(down[1], DMatrix[1][3], true);
                addValue(down[1], DMatrix[2][3], false);
                addValue(down[1], DMatrix[2][2], true);
                addValue(down[1], DMatrix[3][2], false);
                addValue(down[1], DMatrix[3][1], true);
                str = "Si raccolgono gli oggetti della diagonale 3.";
                break;
                
            case 5:
                addValue(down[0], DMatrix[0][3], true);
                addValue(down[0], DMatrix[1][3], false);
                addValue(down[0], DMatrix[1][2], true);
                addValue(down[0], DMatrix[2][2], false);
                addValue(down[0], DMatrix[2][1], true);
                addValue(down[0], DMatrix[3][1], false);
                addValue(down[0], DMatrix[3][0], true);
                str = "Si raccolgono gli oggetti della diagonale 4.";
                break;
                
            case 6:
                addValue(left[3], DMatrix[0][3], false);
                addValue(left[3], DMatrix[0][2], true);
                addValue(left[3], DMatrix[1][2], false);
                addValue(left[3], DMatrix[1][1], true);
                addValue(left[3], DMatrix[2][1], false);
                addValue(left[3], DMatrix[2][0], true);
                addValue(left[3], DMatrix[3][0], false);
                str = "Si raccolgono gli oggetti della diagonale 5.";
                break;
            
            case 7:
                addValue(left[2], DMatrix[0][2], false);
                addValue(left[2], DMatrix[0][1], true);
                addValue(left[2], DMatrix[1][1], false);
                addValue(left[2], DMatrix[1][0], true);
                addValue(left[2], DMatrix[2][0], false);
                str = "Si raccolgono gli oggetti della diagonale 6.";
                break;
                
            case 8:
                addValue(left[1], DMatrix[0][1], false);
                addValue(left[1], DMatrix[0][0], true);
                addValue(left[1], DMatrix[1][0], false);
                str = "Si raccolgono gli oggetti della diagonale 7.";
                break;
            
            case 9:
                addValue(left[0], DMatrix[0][0], false);
                str = "Si raccolgono gli oggetti della diagonale 8.";
                break;

            case 10:
                this.convertResult();
                str = "Si convertono gli oggetti del risultato.";
                str += "\nIl risultato della moltiplicazione "+multiplying+"*"+multiplier+" è "+this.getResult()+".";
                break;
        }
    }
    
    /**
     * Metodo ausiliario per lo svolgimento della divisione.
     * Questo metodo provvede alla divisione tra la colonna <code>col</code> di SMatrix 
     * e il divisore inserendo il risultato in up[<code>col</code>].
     * Inoltre tale metodo provvede alla descrizione dello step in corso.
     * @param col 
     */
    private void divCol(int col){
        
        int valueCol = this.getColumn(col);
        int valueRight = this.getValueOf(right);
        int result = valueCol / valueRight;
        
        this.resetColumn(col);
        
        up[col].setValue(result);
        
        if(col < 3){  
            int rest = (valueCol % valueRight)*10 + this.getColumn(col+1);
            this.setColumn(col+1, rest);
            
        } else {
            int rest = valueCol % valueRight;
            this.setColumn(col, rest);
        }
        
        int cell = 4-col;
       
        if(result == 0) str = "Non viene tolta nessuna copia del divisore dalla colonna "+cell+". Quindi viene inserito 1 Caracol nella cella "+cell+" del risultato in alto.";
        else if(result == 1) str = "Viene tolta 1 copia del divisore dalla colonna "+cell+". Quindi viene inserito 1 Frijolito nella cella "+cell+" del risultato in alto.";
        else str = "Vengono tolte "+result+" copie del divisore dalla colonna "+cell+". Il valore del quoziente viene inserito nella colonna "+cell+" del risultato in alto.";
            
        if(col < 3) str += "\nGli eventuali oggetti rimasti nella colonna "+cell+" vengono trasportati nella colonna a destra e spostati di una casella in alto per preservarne la significatività.";
    }

    /**
     * Metodo da usare per lo svolgimento della divisione a step.
     * Questo metodo deve essere richiamato passando come parametro <code>n</code>
     * il numero dello step da eseguire. Gli steps per questa operazione vanno da 0 a 5.
     * Per il corretto svolgimento dell'operazione, ad ogni nuovo step da eseguire
     * (che sia superiore o inferiore), tale metodo deve essere richiamato per tutti
     * gli steps precedenti ad <code>n</code> partendo da 0 e proseguendo di 1
     * step alla volta.
     * Lo step 0 prepara l'abaco per lo svolgimento dell'operazione.
     * Lo step 1 copia gli oggetti di <code>down</code> in SMatrix[3][*].
     * Gli steps da 2 a 5 provvedono allo svolgimento della divisione colonna per colonna.
     * @param n lo step da eseguire
     */
    public void div(int n){

        int dividend = this.getValueOf(down);
        int divisor = this.getValueOf(right);
        
        switch(n){
            case 0:
                str = "";
                this.resetSMatrix();
                this.resetOperand(up);
                break;
            case 1:
                int length = 4 - Integer.toString(this.getValueOf(down)).length();
                for(int i=3; i>=length; i--)
                    SMatrix[3][i].setValue(down[i].getValue());
                
                str = "Gli oggetti del dividendo vengono inseriti nella riga 1 dell'abaco.";
                break;
            
            case 2:
                this.divCol(0);
                break;
                
            case 3:
                this.divCol(1);
                break;
                
            case 4:
                this.divCol(2);
                break;
                
            case 5:
                this.divCol(3);
                str += "\nGli eventuali oggetti rimasti sono il resto della divisione.";
                str += "\nIl risultato della divisione "+dividend+"/"+divisor+" è "+this.getValueOf(up)+" con resto "+this.getColumn(3)+".";
                break;  
        }
    }
}