package abacomayadecimale;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.KeyEventDispatcher;
import java.awt.KeyboardFocusManager;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Random;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextArea;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

/**
 * Classe per l'implementazione grafica dell'abaco maya in forma decimale.
 * Permette l'esecuzione di addizioni, sottrazioni, moltiplicazioni e divisioni in
 * due differenti modalità:
 * <ul>
 * <li>Simulatore: permette all'utente di scegliere l'operazione da effettuare,
 * impostare gli operandi, visualizzare step by step l'esecuzione dell'operazione
 * per permette un miglior apprendimento dell'argomento.
 * <li>Esercitazione: permette all'utente di scegliere l'operazione, il livello
 * da affrontare e di procedere alla realizzazione dell'operazione direttamente
 * sull'abaco. Infine l'utente può verificare la correttezza dell'esercizio svolto
 * e correggere gli errori commessi per un miglior apprendimento.
 * </ul>
 */
public class AbacoFrame extends JFrame implements ActionListener, ItemListener, KeyEventDispatcher, MouseListener, MouseMotionListener {

    /**
     * Costante per determinare se l'abaco è in modalità simulatore.
     */
    public final static int SIM = 0;
    
    /**
     * Costante per determinare se l'abaco è in modalità esercitazione.
     */
    public final static int EXE = 1;

    /**
     * Modalità corrente selezionata nell'abaco.
     * La variabile è statica per permettere alle altre classi (SCell e DCell)
     * di conoscere la modalità scelta e comportarsi di conseguenza.
     * @see AbacoFrame#SIM
     * @see AbacoFrame#EXE
     */
    public static int currentModality;
    
    /**
     * Operazione corrente selezionata nell'abaco.
     * La variabile è statica per permettere alle altre classi (SCell e DCell)
     * di conoscere la modalità scelta e comportarsi di conseguenza.
     * @see Abaco#SUM
     * @see Abaco#SUB
     * @see Abaco#MUL
     * @see Abaco#DIV
     */
    public static int currentOperation;
    
    /**
     * Cursore corrente selezionato nell'abaco.
     */
    private CursorButton currentCursorButton;
    
    /**
     * Valore del primo operando.
     * Questa variabile viene utilizzata:
     * <ul>
     * <li>Addizione: come primo addendo
     * <li>Sottrazione: come minuendo
     * <li>Moltiplicazione: come moltiplicando
     * <li>Divisione: come dividendo
     * </ul>
     */
    private int valueOne = 0;
    
    /**
     * Valore del secondo operando.
     * Questa variabile viene utilizzata:
     * <ul>
     * <li>Addizione: come secondo addendo
     * <li>Sottrazione: come sottraendo
     * <li>Moltiplicazione: come moltiplicatore
     * <li>Divisione: come divisore
     * </ul>
     */
    private int valueTwo = 0;
    
    /**
     * Valore del secondo operando.
     * Questa variabile viene utilizzata solo nell'addizione come terzo addendo.
     * </ul>
     */
    private int valueThree = 0;
    
    /**
     * Riferimento all'abaco logico.
     * @see Abaco
     */
    private Abaco abaco;
    
    /**
     * Valore step corrente.
     */
    private int currentStep;
    
    /**
     * Valore dello step massimo.
     */
    private int maxStep;

    /**
     * Pannello principale che contiene tutti gli oggetti grafici.
     */
    private JPanel mainPanel;
    
    /**
     * Pannello per centrare orizontalmente il pannello <code>graphics</code>.
     */
    private JPanel graphicsPanel;
    
    /**
     * Pannello per centrare orizontalmente il pannello <code>menu</code>.
     */
    private JPanel menuPanel;
    
    /**
     * Pannello che contiene le celle dell'abaco grafico.
     */
    private JPanel graphics;
    
    /**
     * Pannello che contiene i pannelli del menu.
     */
    private JPanel menu;
    
    /**
     * Pannello che contiene le celle di <code>SMatrix</code>.
     * @see Abaco#SMatrix
     */
    private JPanel sPanel;
    
    /**
     * Pannello che contiene le celle di <code>DMatrix</code>.
     * @see Abaco#DMatrix
     */
    private JPanel dPanel;

    /**
     * Pannello che contiene le celle di <code>up</code>.
     * @see Abaco#up
     */
    private JPanel u;
    
    /**
     * Pannello che contiene le celle di <code>right</code>.
     * @see Abaco#right
     */
    private JPanel r;
    
    /**
     * Pannello che contiene le celle di <code>down</code>.
     * @see Abaco#down
     */
    private JPanel d;
    
    /**
     * Pannello che contiene le celle di <code>left</code>.
     * @see Abaco#left
     */
    private JPanel l;
    
    /**
     * Pannello del menu necessario per la modalità simulatore.
     * Il pannello contiene il numero di step corrente e massimo, i bottoni per
     * andare allo step successivo o precedente.
     */
    private JPanel stepsPanel;
    
    /**
     * Pannello del menu necessario per la modalità esercitazione.
     * Il pannello contiene una combobox per la scelta del livello e bottoni per
     * creare un nuovo esercizio e per la verifica.
     */
    private JPanel exercisePanel;
    
    /**
     * Bottone per selezionare il cursore "default".
     */
    private CursorButton defaultButton;  
    
    /**
     * Bottone per selezionare il cursore "cross".
     */
    private CursorButton crossButton;
    
    /**
     * Bottone per selezionare il cursore "help".
     */
    private CursorButton helpButton;
    
    /**
     * Bottone per selezionare il cursore "caracol".
     */
    private CursorButton caracolButton;
    
    /**
     * Bottone per selezionare il cursore "frijolito".
     */
    private CursorButton frijolitoButton;
    
    /**
     * Bottone per selezionare il cursore "palito".
     */
    private CursorButton palitoButton;
    
    /**
     * JRadioButton per selezionare la modalità simulatore.
     */
    private JRadioButton simulator;
    
    /**
     * JRadioButton per selezionare la modalità esercitazione.
     */
    private JRadioButton exercise;
    
    /**
     * JRadioButton per selezionare l'operazione addizione.
     */
    private JRadioButton sum;
    
    /**
     * JRadioButton per selezionare l'operazione sottrazione.
     */
    private JRadioButton sub;
    
    /**
     * JRadioButton per selezionare l'operazione moltiplicazione.
     */
    private JRadioButton mul;
    
    /**
     * JRadioButton per selezionare l'operazione divisione.
     */
    private JRadioButton div;
    
    /**
     * Bottone per avviare il calcolo step by step.
     */
    private JButton calc;
    
    /**
     * Bottone per togliere tutti gli oggetti dall'abaco.
     * Nella modalità esercitazione toglie tutti gli oggetti tranne quelli degli
     * operandi.
     */
    private JButton reset;
    
    /**
     * Label per la visualizzazione dello step corrente e massimo.
     */
    private JLabel step;
    
    /**
     * Bottone per andare allo step precedente.
     */
    private JButton prev;
    
    /**
     * Bottone per andare allo step successivo.
     */
    private JButton next;
    
    /**
     * ComboBox per la scelta del livello di difficoltà dell'esercizio.
     * I livelli sono Facile, Medio e Difficile.
     */
    private JComboBox<String> level;
    
    /**
     * Bottone per creare un nuovo esercizio.
     */
    private JButton newExercise;
    
    /**
     * Bottone per la verifica dell'esercizio.
     */
    private JButton check;
    
    /**
     * Area di testo riservata alla descrizione dello step corrente.
     */
    private JTextArea description;
    
    /**
     * Splashscreen per dare il benvenuto all'utente.
     */
    private Splash splash;
    
    /**
     * Tip per il suggerimento nella modalità esercitazione.
     */
    private Tip tip;
    
    
    /**
     * Costruttore dell'oggeto AbacoFrame.
     * Il costruttore provvede alla creazione della finestra di dialogo che permette
     * l'interazione con l'utente. Vengono creati tutti i pannelli necessari al 
     * corretto posizionamento di tutti i componenti (celle, bottoni, ecc.).
     * Il frame avrà una grandezza di 800x550 px, l'utente sarà impossibilitato
     * a cambiarne la dimenzione. La finestra si troverà al centro dello schermo
     * (ove possibile). I componenti avranno l'aspetto dei controlli del sistema
     * operativo in uso.
     */
    public AbacoFrame(){
        super();

        //Inizializza i dati
        this.abaco = new Abaco();
        this.currentStep = 0;
        this.maxStep = 0;
        
        //Inizializza i pannelli
        mainPanel = new JPanel(new BorderLayout());
        this.createGraphicsPanel();
        this.createMenuPanel();
        
        //Crea la label per la descrizione dello step
        description = new JTextArea();
        description.setOpaque(false);
        description.setEditable(false);
        description.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        description.setLineWrap(true);
        description.setWrapStyleWord(true);
        
        //Sistema i pannelli
        mainPanel.add(graphicsPanel, BorderLayout.CENTER);
        mainPanel.add(menuPanel, BorderLayout.EAST);
        mainPanel.add(description, BorderLayout.SOUTH);
        this.getContentPane().add(mainPanel);
        
        //Splashscreen iniziale per dare il benvenuto
        splash = new Splash("splash", new Dimension(800, 550));
        this.getLayeredPane().add(splash);
        splash.addMouseListener(this);
                        
        //Imposta il titolo
        this.setTitle("Abaco Maya Decimale v1.0");
        //Imposto il cursore predefinito
        this.currentCursorButton = defaultButton;
        this.setCursor(defaultButton.getFirstCursor());
        //Imposto il il KeyEventDispatcher per gestire l'input da tastiera in tutti i componenti
        KeyboardFocusManager.getCurrentKeyboardFocusManager().addKeyEventDispatcher(this);
        //Ricrea la grafica del sistema operativo usato
        try { UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName()); } catch (ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException ex){}
        //Imposta la grandezza del JFrame
        this.setSize(800, 550);
        this.setResizable(false);
        //Sistema la posizione della finestra
        if(Toolkit.getDefaultToolkit().getScreenSize().height > 600)
            this.setLocationRelativeTo(null);
        //Imposta l'operazione alla chiusura del JFrame
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);        
        //Rende visibile la finestra
        this.setVisible(true);
    }

    /**
     * Crea l'abaco graficamente.
     */
    private void createGraphicsPanel(){
        //Inizializza i pannelli
        graphicsPanel = new JPanel();
        graphics = new JPanel(new BorderLayout());
        sPanel = new JPanel(new GridLayout(4, 4));
        dPanel = new JPanel(new GridLayout(4, 4));
        u = new JPanel();
        r = new JPanel();
        d = new JPanel();
        l = new JPanel();
        
        u.setLayout(new BoxLayout(u, BoxLayout.X_AXIS));
        r.setLayout(new BoxLayout(r, BoxLayout.Y_AXIS));
        d.setLayout(new BoxLayout(d, BoxLayout.X_AXIS));
        l.setLayout(new BoxLayout(l, BoxLayout.Y_AXIS));
        
        //Imposto i bordi trasperenti ai pannelli per distanziarli da "sPanel" o "dPanel"
        sPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        dPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        r.setBorder(BorderFactory.createEmptyBorder(5, 0, 0, 0));
        l.setBorder(BorderFactory.createEmptyBorder(5, 0, 0, 0));
        
        u.add(Box.createHorizontalGlue());
        d.add(Box.createHorizontalGlue());
        //Aggiunge le celle ai pannelli
        for(int i = 0; i<4; i++){
            for(int j = 0; j<4; j++){
                sPanel.add(abaco.SMatrix[i][j]);
                dPanel.add(abaco.DMatrix[i][j]);
                
                abaco.SMatrix[i][j].addMouseListener(this);
                abaco.DMatrix[i][j].addMouseListener(this);
                abaco.SMatrix[i][j].addMouseMotionListener(this);
                abaco.DMatrix[i][j].addMouseMotionListener(this);
            }
            u.add(abaco.up[i]);
            r.add(abaco.right[i]);
            d.add(abaco.down[i]);
            l.add(abaco.left[i]);
            
            abaco.up[i].addMouseListener(this);
            abaco.right[i].addMouseListener(this);
            abaco.down[i].addMouseListener(this);
            abaco.left[i].addMouseListener(this);
            
            abaco.up[i].addMouseMotionListener(this);
            abaco.right[i].addMouseMotionListener(this);
            abaco.down[i].addMouseMotionListener(this);
            abaco.left[i].addMouseMotionListener(this);
        }
        u.add(Box.createHorizontalGlue());
        d.add(Box.createHorizontalGlue());
        
        //Aggiunge i pannelli a "graphics"
        graphics.add(u, BorderLayout.NORTH);
        graphics.add(r, BorderLayout.EAST);
        graphics.add(d, BorderLayout.SOUTH);
        graphics.add(l, BorderLayout.WEST);
        graphics.add(sPanel, BorderLayout.CENTER);

        //Aggiunge "graphics" al pannello "graphicsPanel"
        graphicsPanel.add(graphics);  
    }
    
    /**
     * Crea il menu graficamente.
     */
    private void createMenuPanel(){
        
        //Inizializza i componenti
        defaultButton = new CursorButton("default", new Point(12, 10));
        crossButton = new CursorButton("cross", new Point(16, 16));
        helpButton = new CursorButton("help", new Point(16, 16));
        caracolButton = new CursorButton("caracol", new Point(16, 16));
        frijolitoButton = new CursorButton("frijolito", new Point(16, 16));
        palitoButton = new CursorButton("palito", new Point(16, 16));
        simulator = new JRadioButton("Simulatore");
        exercise = new JRadioButton("Esercitazione");
        sum = new JRadioButton("<html><u>A</u>ddizione</html>");
        sub = new JRadioButton("<html><u>S</u>ottrazione</html>");
        mul = new JRadioButton("<html><u>M</u>oltiplicazione</html>");
        div = new JRadioButton("<html><u>D</u>ivisione</html>");
        calc = new JButton("Calcola");
        reset = new JButton("Reset");
        step = new JLabel("Step 0 / 0", JLabel.CENTER);
        prev = new JButton("Prec.");
        next = new JButton("Succ.");
        level = new JComboBox<>();
        level.addItem("Facile");
        level.addItem("Medio");
        level.addItem("Difficile");
        newExercise = new JButton("Nuovo");
        check = new JButton("Verifica");
        
        helpButton.setEnabled(false);

        //Crea un ButtonGroup per far selezionare solo una modalità
        ButtonGroup modGroup = new ButtonGroup();
        modGroup.add(simulator);
        modGroup.add(exercise);
        
        //Crea un ButtonGroup per far selezionare solo una delle 4 operazioni
        ButtonGroup opGroup = new ButtonGroup();
        opGroup.add(sum);
        opGroup.add(sub);
        opGroup.add(mul);
        opGroup.add(div);
        
        //Aggiunge gli ascoltatori
        //CursorButtons
        defaultButton.addActionListener(this);
        crossButton.addActionListener(this);
        helpButton.addActionListener(this);
        caracolButton.addActionListener(this);
        frijolitoButton.addActionListener(this);
        palitoButton.addActionListener(this);
        //Modalità
        simulator.addItemListener(this);
        exercise.addItemListener(this);
        //Operazioni
        sum.addItemListener(this);
        sub.addItemListener(this);
        mul.addItemListener(this);
        div.addItemListener(this);
        //Steps 
        calc.addActionListener(this);
        reset.addActionListener(this);
        prev.addActionListener(this);
        next.addActionListener(this);
        //Esercizio
        level.addItemListener(this);
        newExercise.addActionListener(this);
        check.addActionListener(this);
        
        //Disabilita "prev" e "next"
        prev.setEnabled(false);
        next.setEnabled(false);
        
        //Crea e inizializza i pannelli per il menu
        menuPanel = new JPanel();
        menu = new JPanel();
        menu.setLayout(new BoxLayout(menu, BoxLayout.Y_AXIS));
                
        //CursorsButton
        JPanel cursors = new JPanel();
        cursors.setLayout(new BoxLayout(cursors, BoxLayout.Y_AXIS));
        JPanel cursorsOne = new JPanel();   
        JPanel cursorsTwo = new JPanel();
        //Modality
        JPanel modality = new JPanel(new GridLayout(2, 1));
        //Operations
        JPanel operations = new JPanel();
        operations.setLayout(new BoxLayout(operations, BoxLayout.Y_AXIS));
        JPanel radiosOperations = new JPanel(new GridLayout(4, 1));
        JPanel buttonsOperations = new JPanel();
        //Steps
        stepsPanel = new JPanel(new BorderLayout());
        //Exercise
        exercisePanel = new JPanel(new BorderLayout());        
        
        //Imposta i bordi dei pannelli
        cursors.setBorder(BorderFactory.createTitledBorder("Cursori"));
        modality.setBorder(BorderFactory.createTitledBorder("Modalità"));
        operations.setBorder(BorderFactory.createTitledBorder("Operazioni"));
        stepsPanel.setBorder(BorderFactory.createTitledBorder("Steps"));
        exercisePanel.setBorder(BorderFactory.createTitledBorder("Esercizio"));
        
        //Sistema i componenti nei pannelli
        //CursorButtons
        cursorsOne.add(defaultButton);
        cursorsOne.add(crossButton);
        cursorsOne.add(helpButton);
        cursorsTwo.add(palitoButton);
        cursorsTwo.add(frijolitoButton);
        cursorsTwo.add(caracolButton);
        
        cursors.add(cursorsOne);
        cursors.add(cursorsTwo);
        
        //Modalità
        modality.add(simulator);
        modality.add(exercise);
        
        //Operazioni
        buttonsOperations.add(calc);
        buttonsOperations.add(reset);
        radiosOperations.add(sum);
        radiosOperations.add(sub);
        radiosOperations.add(mul);
        radiosOperations.add(div);
        operations.add(radiosOperations);
        operations.add(buttonsOperations);

        //Steps
        stepsPanel.add(step, BorderLayout.NORTH);
        stepsPanel.add(prev, BorderLayout.WEST);
        stepsPanel.add(next, BorderLayout.EAST);
        
        //Esercitazione
        exercisePanel.add(level, BorderLayout.NORTH);
        exercisePanel.add(newExercise, BorderLayout.WEST);
        exercisePanel.add(check, BorderLayout.EAST);
        
        menu.add(cursors);
        menu.add(modality);
        menu.add(operations);
        menu.add(stepsPanel);
        
        //Aggiunge il menu al pannello menuPanel
        menuPanel.add(menu);
    }

    /**
     * Aggiorna graficamente l'abaco.
     */
    private void update(){
        this.revalidate();
        this.repaint();
    }
    
    /**
     * Aggiorna la label <code>step</code>.
     */
    private void updateStep(int current, int max){
        currentStep = current;
        maxStep = max;
        step.setText("Step "+currentStep+" / "+maxStep);
        
        if(currentStep == 0)
            description.setText("");
        else description.setText(abaco.str);
        
        prev.setEnabled(currentStep > 0);
        next.setEnabled(currentStep < maxStep); 
    }
    
    /**
     * Questo metodo permette l'abilitazione/disabilitazione dei bottoni per
     * il cambio del cursore.
     * Ogni volta che viene richiamato il metodo viene impostato il cursore di
     * default dell'applicazione.
     * 
     * @param bool TRUE per abilitare i bottoni, FALSE altrimenti
     */
    private void setCursorsEnabled(boolean bool){
        
        this.cursorButtonClicked(defaultButton);
        
        defaultButton.setEnabled(bool);
        crossButton.setEnabled(bool);
        helpButton.setEnabled(currentModality == EXE);
        caracolButton.setEnabled(bool);
        frijolitoButton.setEnabled(bool);
        palitoButton.setEnabled(bool);
    }
    
    
    /**
     * Reinserisce gli operandi nell'abaco.
     * Gli operandi vengono sistemati nell'abaco in base al tipo di operazione
     * scelta dall'utente.
     */
    private void reinsertOperands(){
        
        switch(this.getOperationSelected()){
            case Abaco.SUM:
                if(valueOne > 0)
                    abaco.setRow(0, valueOne);
                if(valueTwo > 0)
                    abaco.setRow(1, valueTwo);
                if(valueThree > 0)
                    abaco.setRow(2, valueThree);
                break;
                
            case Abaco.SUB:
                if(valueOne > 0)
                    abaco.setRow(0, valueOne);
                if(valueTwo > 0)
                    abaco.setRow(1, valueTwo);
                break;
                
            case Abaco.MUL:
                if(valueOne > 0)
                    abaco.setOperand(abaco.up, valueOne);
                if(valueTwo > 0)
                    abaco.setOperand(abaco.right, valueTwo);
                break;
                
            case Abaco.DIV:
                if(valueOne > 0)
                    abaco.setOperand(abaco.down, valueOne);
                if(valueOne > 0)
                    abaco.setOperand(abaco.right, valueTwo);
                break;
            }
    }
    
    /**
     * Toglie tutti gli oggetti all'interno dell'abaco.
     * Se l'abaco è in modalità esercitazione, gli oggetti deglioperandi verranno 
     * riaggiunti all'abaco.
     */
    private void reset(){
        abaco.reset();
        this.updateStep(0, 0);
        this.setCursorsEnabled(true);
        calc.setEnabled(currentModality == SIM);
        this.setIstructions(currentOperation);
        
        if(currentModality == EXE)
            this.reinsertOperands();
    }
    
    /**
     * Ritorna la costante del tipo di operazione.
     * @see Abaco#SUM
     * @see Abaco#SUB
     * @see Abaco#MUL
     * @see Abaco#DIV
     */
    private int getOperationSelected(){
        
        if(sum.isSelected())
            return Abaco.SUM;
        else if(sub.isSelected())
            return Abaco.SUB;
        else if(mul.isSelected())
            return Abaco.MUL;
        else if(div.isSelected())
            return Abaco.DIV;
        
        return -1;
    }
    
    /**
     * Calcola fino allo step <code>x</code>.
     * Il metodo reinserisce gli operandi, richiama il metodo dell'operazione
     * dallo step 0 fino allo step <code>x</code>, infine aggiorna la label
     * <code>step</code>
     * @param x step da calcolare
     */
    private void calcToStep(int x){
        
        this.reinsertOperands();
        
        switch(this.getOperationSelected()){
            case Abaco.SUM:
                for(int i = 0; i<=x; i++)
                    abaco.sum(i);
                break;
                
            case Abaco.SUB:
                for(int i = 0; i<=x; i++)
                    abaco.sub(i);
                break;
                
            case Abaco.MUL:
                for(int i = 0; i<=x && i<maxStep; i++)
                    abaco.mul(i);
                
                if(x == maxStep)
                    abaco.mul(10);
                
                break;
            
            case Abaco.DIV:   
                abaco.div(0);
                if(x > 0)
                    abaco.div(1);
                
                int dif = 4-Integer.toString(abaco.getValueOf(abaco.down)).length();
                for(int i = 2+dif ; i<=x+dif; i++)
                    abaco.div(i);

                break;
        }
        
        this.updateStep(currentStep, maxStep);
    }
    
    /**
     * Questo metodo gestisce l'evento del clic su <code>calc</code>.
     * Quando viene richiamato il metodo vengono salvati gli operandi nelle rispettive
     * variabili, viene calcolato lo step massimo dell'operazione corrente, vengono
     * attivati i bottoni per permettere all'utente di eseguire l'operazione
     * step by step e viene eseguito lo step 0.
     * 
     * In caso gli operandi non soddisfano i requisiti minimi dell'operazione,
     * l'utente viene avvertito attraverso un messaggio di dialogo creato dalla
     * classe JOptionPane.
     * 
     * @see JOptionPane#showMessageDialog
     */
    private void calc(){
        
        int one, two, three;

        switch(this.getOperationSelected()){
            case Abaco.SUM:
                //Salva i valori inseriti dall'utente
                valueOne = abaco.getRow(0);
                valueTwo = abaco.getRow(1);
                valueThree = abaco.getRow(2);
                
                //Calcola il numero di cifre di ogni valore
                one = String.valueOf(valueOne).length();
                two = String.valueOf(valueTwo).length();
                three = String.valueOf(valueThree).length();
                
                //Calcola il numero di step necessari per l'addizione
                maxStep = 1 + Math.max(Math.max(one, two), three);
                break;
                
            case Abaco.SUB:
                //Salva i valori inseriti dall'utente
                valueOne = abaco.getRow(0);
                valueTwo = abaco.getRow(1);
                valueThree = 0;
                
                //Controlla se l'utente ha inserito valori che permettono di eseguire l'operazione
                if((valueOne-valueTwo) < 0){
                    JOptionPane.showMessageDialog(this, "Gli operandi scelti non permettono la sottrazione sull'Abaco Maya.\nInserisci un sottraendo maggiore o uguale al minuendo.", "", JOptionPane.ERROR_MESSAGE);
                    return;
                }
                
                //Calcola il numero di cifre di ogni valore
                one = String.valueOf(valueOne).length();
                two = String.valueOf(valueTwo).length();
                
                //Calcola il numero di step necessari per la sottrazione
                maxStep = Math.max(one, two);
                break;
                
            case Abaco.MUL:
                //Salva i valori inseriti dall'utente
                valueOne = abaco.getValueOf(abaco.up);
                valueTwo = abaco.getValueOf(abaco.right);
                valueThree = 0;
                
                //Calcola il numero di step necessari per la moltiplicazione
                maxStep = 2 + String.valueOf(valueOne * valueTwo).length();
                break;
                
            case Abaco.DIV:
                //Salva i valori inseriti dall'utente
                valueOne = abaco.getValueOf(abaco.down);
                valueTwo = abaco.getValueOf(abaco.right);
                valueThree = 0;
                
                //Controlla se l'utente ha inserito valori che permettono di eseguire l'operazione
                if(valueTwo == 0){
                    JOptionPane.showMessageDialog(this, "Non si può eseguire una divisione per ZERO.", "", JOptionPane.ERROR_MESSAGE);
                    return;
                }
                
                //Calcola il numero di step necessari per la divisione
                maxStep = 1 + String.valueOf(valueOne).length();
                break;
        }
        
        if(valueOne == 0 && valueTwo == 0 && valueThree == 0){
            JOptionPane.showMessageDialog(this, "Inserisci gli operandi!", "", JOptionPane.ERROR_MESSAGE);
        } else {
            this.setCursorsEnabled(false);
            calc.setEnabled(false);
            calcToStep(currentStep);
        }
    }
    
    /**
     * Gestisce l'evento del clic su <code>button</code>.
     * Se il cursore utilizzato fin ora era quello principale, il nuovo cursore
     * dell'applicazione sarà quello principale del <code>button</code>.
     * Se il cursore utilizzato ifn ora era quello secondario, il nuovo cursore
     * dell'applicazione sarà quello secondario del <code>button</code>.
     * @param button il CursorButton a cui applicare il metodo.
     */
    private void cursorButtonClicked(CursorButton button){
        
        if(this.getCursor() == currentCursorButton.getFirstCursor())
            this.setCursor(button.getFirstCursor());
        else this.setCursor(button.getSecondCursor());
        
        currentCursorButton = button;        
    }
    
    /**
     * Gestisce l'evento del clic su <code>prev</code>. 
     */
    private void prev(){
        this.calcToStep(--currentStep);
    } 
    
    /**
     * Gestisce l'evento del clic su <code>next</code>.
     */
    private void next(){
        this.calcToStep(++currentStep);
    }
    
    /**
     * Crea e restituisce un valore intero casuale.
     * Il valore creato avrà almeno <code>min</code> cifre e massimo <code>max</code>
     * cifre, inoltre le cifre avranno un range da <code>from</code> a <code>to</code>.
     * 
     * <br>Esempio: this.createValue(2, 3, 2, 5);
     * <br>Il metodo restituirà valori con almeno 2 cifre e massimo 3,
     * le cifre invece potranno essere solo 1/2/3/4/5 e non uscirà mai una cifra
     * superiore a 5.
     * <br>Possibili valori restituiti: 423, 4, 55.
     * 
     * @param min numero minimo di cifre
     * @param max numero massimo di cifre
     * @param from valore minimo di ogni cifra
     * @param to valore massimo di ogni cifra
     * @return un intero con il valore casuale creato
     */
    private int createValue(int min, int max, int from, int to){
        
        Random rand = new Random();
        String val = new String();

        int n = rand.nextInt(1+max-min)+min;
        for(int i = 0; i<n; i++){
            val += rand.nextInt(to-from)+from;
        }
        
        int value = Integer.parseInt(val);
        if(value > 1)
            return value;
        
        return createValue(min, max, from, to);        
    }
    
    /**
     * Crea un esercizio per l'addizione in base al livello scelto dall'utente.
     */
    private void createSumExercise(){

        switch(level.getSelectedIndex()){
            case 0:
                valueOne = this.createValue(2, 3, 0, 5);
                valueTwo = this.createValue(1, 3, 0, 5);
                valueThree = 0;
                break;
            
            case 1:
                valueOne = this.createValue(2, 3, 0, 8);
                valueTwo = this.createValue(2, 3, 0, 7);
                valueThree = this.createValue(1, 3, 0, 7);
                break;
              
            case 2:
                valueOne = this.createValue(3, 4, 4, 9);
                valueTwo = this.createValue(2, 4, 3, 9);
                valueThree = this.createValue(2, 4, 3, 9);
                break;
        }
        
        abaco.setRow(0, valueOne);
        abaco.setRow(1, valueTwo);
        
        if(valueThree > 0)
            abaco.setRow(2, valueThree);
    }
    
    
    /**
     * Crea un esercizio per la sottrazione in base al livello scelto dall'utente.
     */
    private void createSubExercise(){
        switch(level.getSelectedIndex()){
            case 0:
                do{
                    valueOne = this.createValue(3, 4, 4, 9);
                    valueTwo = this.createValue(2, 3, 0, 4);
                } while(valueOne - valueTwo < 0);
                break;
            
            case 1:
                do{
                    valueOne = this.createValue(3, 4, 4, 9);
                    valueTwo = this.createValue(2, 4, 2, 7);
                } while(valueOne - valueTwo < 0);
                break;
              
            case 2:
                do{
                    valueOne = this.createValue(3, 4, 4, 9);
                    valueTwo = this.createValue(3, 4, 4, 9);
                } while(valueOne - valueTwo < 0);
                break;
        }
        
        abaco.setRow(0, valueOne);
        abaco.setRow(1, valueTwo);
    }
    
    
    /**
     * Crea un esercizio per la moltiplicazione in base al livello scelto dall'utente.
     */
    private void createMulExercise(){
        switch(level.getSelectedIndex()){
            case 0:
                valueOne = this.createValue(2, 2, 1, 5);
                valueTwo = this.createValue(1, 2, 1, 5);
                break;
            
            case 1:
                valueOne = this.createValue(2, 3, 3, 8);
                valueTwo = this.createValue(2, 3, 2, 7);
                break;
              
            case 2:
                valueOne = this.createValue(3, 4, 5, 9);
                valueTwo = this.createValue(3, 4, 5, 9);
                break;
        }
        
        abaco.setOperand(abaco.up, valueOne);
        abaco.setOperand(abaco.right, valueTwo);
    }
    
    
    /**
     * Crea un esercizio per la divisione in base al livello scelto dall'utente.
     */
    private void createDivExercise(){
        switch(level.getSelectedIndex()){
            case 0:
                do{
                    valueOne = this.createValue(2, 2, 1, 9);
                    valueTwo = this.createValue(1, 2, 1, 5);
                } while(valueOne < valueTwo);
                break;
            
            case 1:
                do{
                    valueOne = this.createValue(2, 3, 3, 9);
                    valueTwo = this.createValue(1, 2, 1, 7);
                } while(valueOne < valueTwo);
                break;
              
            case 2:
                do{
                    valueOne = this.createValue(4, 4, 3, 9);
                    valueTwo = this.createValue(1, 3, 1, 8);
                } while(valueOne < valueTwo);
                break;
        }
        
        abaco.setOperand(abaco.down, valueOne);
        abaco.setOperand(abaco.right, valueTwo);
    }
    
    
    /**
     * Crea un esercizio in base al livello e operazione scelti dall'utente.
     * @see AbacoFrame#createSumExercise() 
     * @see AbacoFrame#createSubExercise() 
     * @see AbacoFrame#createMulExercise() 
     * @see AbacoFrame#createDivExercise() 
     */
    private void createExercise(){
                
        abaco.reset();
        
        switch(this.getOperationSelected()){
            case Abaco.SUM:
                this.createSumExercise();
                break;
            case Abaco.SUB:
                this.createSubExercise();
                break;
            case Abaco.MUL:
                this.createMulExercise();
                break;
            case Abaco.DIV:
                this.createDivExercise();
                break;
        }
    }
    
    /**
     * Controlla se l'utente ha correttamente svolto l'esercizio.
     * Se l'utente ha svolto correttamente l'esercizio viene mostrato un messaggio
     * per complimentarsi con l'utente.
     * In caso contrario viene mostrato un messaggio di errore per avvertire l'utente
     * che non ha eseguito correttamente l'esercizio indicando le celle in cui
     * non c'è il valore corretto. Inoltre viene avvertito se il risultato è
     * gisuto o sbagliato.
     */
    private void checkOperation(){
        
        String error = "";
        
        int x;
        
        switch(this.getOperationSelected()){
            case Abaco.SUM:
                
                x = abaco.getRow(0) + abaco.getRow(1)+ abaco.getRow(2);
                if(abaco.getResult() != x)
                    error = "Il risultato è sbagliato!\n";
                
                for(int i = 3; i>0; i--){
                    x = abaco.SMatrix[0][i].getValue() + abaco.SMatrix[1][i].getValue() + abaco.SMatrix[2][i].getValue();
                    if(x != abaco.SMatrix[3][i].getValue())
                        error += "Valore errato nella cella 1|"+(4-i)+" !\n";
                }
                
                break;
                
            case Abaco.SUB:
                
                x = abaco.getRow(0) - abaco.getRow(1);
                if(abaco.getResult() != x)
                    error = "Il risultato è sbagliato!\n";
                break;
                
            case Abaco.MUL:
                x = abaco.getValueOf(abaco.up) * abaco.getValueOf(abaco.right);
                if(abaco.getResult() != x)
                    error = "Il risultato è sbagliato!\n";
                
                for(int i = 0; i<4; i++){
                    for(int j = 0; j<4; j++){
                        x = abaco.up[j].getValue() * abaco.right[i].getValue();
                        if(abaco.DMatrix[i][j].getValue() != x)
                            error += "Valore errato nella cella "+(4-i)+"|"+(4-j)+" !\n";
                    }
                }
                
                break;
                
            case Abaco.DIV:
                
                if (abaco.getValueOf(abaco.down)/abaco.getValueOf(abaco.right) != abaco.getValueOf(abaco.up))
                    error = "Il risultato è sbagliato!";
                
                if(abaco.getValueOf(abaco.down)%abaco.getValueOf(abaco.right) != abaco.getColumn(3))
                    error += "\nIl resto della divisione è sbagliato!";
                
                break;
        }
        
        if(!error.equals(""))
            JOptionPane.showMessageDialog(this, error, null, JOptionPane.ERROR_MESSAGE);
        else JOptionPane.showMessageDialog(this, "Complimenti! \nSei riuscito a risolvere correttamente l'esercizio!", null, JOptionPane.PLAIN_MESSAGE, new ImageIcon(this.getClass().getResource("images/ok.png")));
    }
    
    /**
     * Ascoltatore per i CurosrButtons e i JButtons.
     */
    @Override
    public void actionPerformed(ActionEvent e) {
            
        JButton button = (JButton)e.getSource();

        if(button instanceof CursorButton)
            this.cursorButtonClicked((CursorButton)button);
        
        else if(button == calc)
            this.calc();

        else if(button == reset)
            this.reset();

        else if(button == prev)
            this.prev();

        else if(button == next)
            this.next();
        
        else if(button == newExercise)
            this.createExercise();
        
        else if(button == check)
            this.checkOperation();
        
        this.update();
    }

    /**
     * Questo metodo provvede alla sostituzione del pannello nel menù in base
     * alla modalità scelta dall'utente.
     */
    private void switchPanelStepsExercise(){
        if(currentModality == SIM){
            menu.remove(exercisePanel);
            menu.add(stepsPanel);
            
            calc.setEnabled(true);
        } else {
            menu.remove(stepsPanel);
            menu.add(exercisePanel);
            
            calc.setEnabled(false);
        }
        
        this.reset();
    }
    
    /**
     * Provvede alla sostituzione dell'abaco grafico se è necessario.
     * Per permettere lo svolgimento della moltiplicazione nell'abaco è necessario
     * cambiare l'abaco a celle singole (<code>sPanel</code>) con l'abaco a celle doppie (<code>dPanel</code>).
     * Se è necessario questo metodo provvede alla sostituzione dell'<code>sPanel</code>
     * con il <code>dPanel</code>.
     * 
     * @param panel il pannello da inserire nell'abaco
     */
    private void changeAbacoWith(JPanel panel){
        boolean bool = true;
        for(Component c : graphics.getComponents())
            if(c.equals(panel))
                bool = false;

        if(bool){
            if(panel.equals(sPanel))
                graphics.remove(dPanel);
            else graphics.remove(sPanel);
            
            graphics.add(panel);
        }

        this.reset();
    }
    
    /**
     * Questo metodo imposta su <code>description</code> una stringa per dare
     * istruzioni all'utente per svolgere le operazioni.
     * @param op l'operazione per la quale dare istruzioni
     * 
     * @see Abaco#SUM
     * @see Abaco#SUB
     * @see Abaco#MUL
     * @see Abaco#DIV
     * 
     */
    private void setIstructions(int op){
        
        String desc = "";
        
        if(currentModality == AbacoFrame.SIM){
            switch(op){
                case Abaco.SUM:
                    desc = "Aggiungi fino a 3 addendi nelle celle evidenziate e clicca su Calcola per iniziare la simulazione.";
                    break;

                case Abaco.SUB:
                    desc = "Aggiungi nella riga 4 il minuendo, nella riga 3 il sottraendo e infine clicca su Calcola per iniziare la simulazione.\n";
                    desc += "Ricorda che nell'abaco Maya non è previsto un risultato negativo.";
                    break;

                case Abaco.MUL:
                    desc = "Aggiungi in alto il moltiplicando, a destra il moltiplicatore e infine clicca su Calcola per iniziare la simulazione.";
                    break;

                case Abaco.DIV:
                    desc = "Aggiungi in basso il dividendo, a destra il divisore e infine clicca su Calcola per iniziare la simulazione.\n";
                    desc += "Ricorda che non si può eseguire una divisione per 0.";
                    break;
            }
        } else {
            switch(op){
                case Abaco.SUM:
                    desc = "Prova a risolvere tu la Somma!.\n";
                    break;

                case Abaco.SUB:
                    desc = "Prova a risolvere tu la Sottrazione.\n";
                    desc += "La riga 4 dell'abaco può essere modificata solo per facilitarti lo svolgimento, pertanto non viene verificata dall'applicazione.\n";
                    break;

                case Abaco.MUL:
                    desc = "Prova a risolvere tu la Moltiplicazione.\n";
                    break;

                case Abaco.DIV:
                    desc = "Prova a risolvere tu la Divisione.\n";
                    
                    break;
            }
            
            desc += "Se non ricordi bene come si svolge, torna al simulatore!";
        }
        
        description.setText(desc);
    }
    
    /**
     * Ascoltatore degli JRadioButtons.
     */
    @Override
    public void itemStateChanged(ItemEvent e) {
        
        if(e.getStateChange() == ItemEvent.SELECTED){
            Object obj = e.getItem();
        
            if(obj instanceof String){
                this.createExercise();

            } else {
                JRadioButton radio = (JRadioButton)obj;
                
                if(radio == simulator){
                    currentModality = SIM;
                    this.switchPanelStepsExercise();
                    
                } else if (radio == exercise){
                    currentModality = EXE;
                    this.switchPanelStepsExercise();
                    
                } else if(radio == sum){
                    currentOperation = Abaco.SUM;
                    this.changeAbacoWith(sPanel);
                    this.setIstructions(currentOperation);
                    
                } else if(radio == sub){
                    currentOperation = Abaco.SUB;
                    this.changeAbacoWith(sPanel);
                    this.setIstructions(currentOperation);
                    
                } else if(radio == mul){
                    currentOperation = Abaco.MUL;
                    this.changeAbacoWith(dPanel);
                    this.setIstructions(currentOperation);
                    
                } else if(radio == div){
                    currentOperation = Abaco.DIV;
                    this.changeAbacoWith(sPanel);
                    this.setIstructions(currentOperation);
                }     
                
                if(currentModality == EXE)
                    this.createExercise();
            }
            
            this.update();
        }
    }
    
    /**
     * Scambia le icone nei CursorButtons. 
     */
    private void switchCursorButtonsIcon(){
        caracolButton.switchIcon();
        frijolitoButton.switchIcon();
        palitoButton.switchIcon();
        
        if(this.getCursor() == currentCursorButton.getFirstCursor())
            this.setCursor(currentCursorButton.getSecondCursor());
        else this.setCursor(currentCursorButton.getFirstCursor());
    }
    
    /**
     * Ascoltatore della tastiera.
     */
    @Override
    public boolean dispatchKeyEvent(KeyEvent e) {
        
        switch(e.getID()){
            case KeyEvent.KEY_RELEASED:
                if(e.getKeyCode() == 18)
                    this.switchCursorButtonsIcon();
                break;
              
            case KeyEvent.KEY_TYPED:
                char key = e.getKeyChar();
                                                
                if(key == 'a' || key == 'A')
                    sum.setSelected(true);
                else if(key == 's' || key == 'S')
                    sub.setSelected(true);
                else if(key == 'm' || key == 'M')
                    mul.setSelected(true);
                else if(key == 'd' || key == 'D')
                    div.setSelected(true);
                else if(key == 'c' || key == 'C')
                    this.cursorButtonClicked(caracolButton);
                else if(key == 'f' || key == 'F')
                    this.cursorButtonClicked(frijolitoButton);
                else if(key == 'p' || key == 'P')
                    this.cursorButtonClicked(palitoButton);

                break;
        }
        return true;
    }
    
    /**
     * Ritorna il valore che dovrebbe assumere la cella indicata dalle coordinate
     * all'interno di <code>SMatrix</code>.
     * @param row riga della cella
     * @param col colonna della cella
     * @return  un intero con il valore che dovrebbe avere la cella
     * 
     * @see Abaco#SMatrix
     */
    private int getCorrectValueAbaco(int row, int col){

        int value = 0;
        
        switch(this.getOperationSelected()){
            case Abaco.SUM:
                value = abaco.SMatrix[0][col].getValue()+abaco.SMatrix[1][col].getValue()+abaco.SMatrix[2][col].getValue();
                break;
                
            case Abaco.SUB:
                break;
                
            case Abaco.MUL:
                break;
                
            case Abaco.DIV:
                //Sistemare il codice
                int res = 0;
                for(int i = 0; i<=col; i++)
                    res += abaco.down[i].getValue()*Math.pow(10, col-i);

                res = res%abaco.getValueOf(abaco.right);

                String s = String.valueOf(res);
                if(s.length() > 3-row)
                    value = s.charAt(s.length()-(3-row)-1)-48;                
                
                break;
        }
        
        return value;
    }
    
    /**
     * Ritorna il valore che dovrebbe assumere la cella indicata dalle coordinate
     * all'interno di up.
     * @param row riga della cella
     * @param col colonna della cella
     * @return  un intero con il valore che dovrebbe avere la cella
     * 
     * @see Abaco#up
     */
    private int getCorrectValueUp(int row, int col){
        
        int value = 0;
        int res;
        String s;
        
        
        switch(this.getOperationSelected()){
            case Abaco.SUM:
                break;
            case Abaco.SUB:
                break;
            case Abaco.MUL:
                break;
            case Abaco.DIV:
                
                res = abaco.getValueOf(abaco.down) / abaco.getValueOf(abaco.right);
                
                s = String.valueOf(res);
                
                if(s.length() > 3-col)
                    value = s.charAt(s.length() -(3-col)-1)-48;
                
                break;
        }
        return value;
    }
     
    /*
     * Metodo non necessario per il corretto funzionamento.
     * Messo solo per completezza anche se commentato per non essere compilato.
    private int getCorrectValueRight(int row, int col){
        switch(this.getOperationSelected()){
            case Abaco.SUM:
                break;
            case Abaco.SUB:
                break;
            case Abaco.MUL:
                break;
            case Abaco.DIV:
                break;
        }
        return 0;
    }
    */
      
    
    /**
     * Ritorna il valore che dovrebbe assumere la cella indicata dalle coordinate
     * all'interno di down.
     * @param row riga della cella
     * @param col colonna della cella
     * @return  un intero con il valore che dovrebbe avere la cella
     * 
     * @see Abaco#down
     */
    private int getCorrectValueDown(int row, int col){
        
        int value = 0;
        int res;
        String s;
        
        switch(this.getOperationSelected()){
            case Abaco.SUM:
                
                res = abaco.getRow(0) + abaco.getRow(1) + abaco.getRow(2);
                
                s = String.valueOf(res);
                               
                if(s.length() > 3-col)
                    value = s.charAt(s.length()-(3-col)-1)-48;
                
                break;
                
            case Abaco.SUB:
                
                res = abaco.getRow(0) - abaco.getRow(1);
                
                s = String.valueOf(res);
                               
                if(s.length() > 3-col)
                    value = s.charAt(s.length()-(3-col)-1)-48;
                break;
                
            case Abaco.MUL:
                
                res = abaco.getValueOf(abaco.up) * abaco.getValueOf(abaco.right);
                
                s = String.valueOf(res);
                               
                if(s.length() > 3-col)
                    value = s.charAt(s.length()-(3-col)-1)-48;
                
                break;
                
            case Abaco.DIV:
                break;
        }
        return value;
    }
       
    
    /**
     * Ritorna il valore che dovrebbe assumere la cella indicata dalle coordinate
     * all'interno di left.
     * @param row riga della cella
     * @param col colonna della cella
     * @return  un intero con il valore che dovrebbe avere la cella
     * 
     * @see Abaco#left
     */
    private int getCorrectValueLeft(int row, int col){
        
        int value = 0;
        int res;
        String s;
        
        switch(this.getOperationSelected()){
            case Abaco.SUM:
                
                res = abaco.getRow(0) + abaco.getRow(1) + abaco.getRow(2);
                
                s = String.valueOf(res);
                                              
                if(s.length()-4 > 3-row)
                    value = s.charAt(s.length()-(3-row)-1-4)-48;
                               
                break;
                
            case Abaco.SUB:
                break;
                
            case Abaco.MUL:
                
                res = abaco.getValueOf(abaco.up) * abaco.getValueOf(abaco.right);
                
                s = String.valueOf(res);
                               
                if(s.length()-4 > 3-row)
                    value = s.charAt(s.length()-(3-row)-1-4)-48;
                
                break;
                
            case Abaco.DIV:
                break;
        }
        return value;
    }
    
    /**
     * Ritorna il valore che dovrebbe assumere la cella <code>cell</code>.
     * @param cell cella di cui si vuole sapere il valore che dovrebbe assumere
     * @return un intero con il valore che dovrebbe assumere la cella
     */
    public int getCorrectValue(Cell cell){
        int row = cell.getRow();
        int col = cell.getCol();

        if(cell instanceof SCell){
            switch(((SCell)cell).getType()){
                case SCell.ABACO:
                    return this.getCorrectValueAbaco(row, col);

                case SCell.UP:
                    return this.getCorrectValueUp(row, col);

                /* Caso non necessario per il corretto funzionamento.
                 * Messo solo per completezza anche se commentato per non essere compilato.
                case SCell.RIGHT:
                    return this.getCorrectValueRight(row, col);
                */
                    
                case SCell.DOWN:
                    return this.getCorrectValueDown(row, col);

                case SCell.LEFT:
                    return this.getCorrectValueLeft(row, col);
            }

        } else {
            return abaco.up[col].getValue() * abaco.right[row].getValue();
        }
        
        return 0;
    }
    
    /**
     * Ascoltatore per gestire il clic del mouse all'interno della cella.
     * In questo metodo viene creato il tip necessario all'aiuto dell'utente.
     * Il tip viene creato solo nella modalità esercitazione e se viene usato il
     * cursore "help".
     */
    @Override
    public void mouseClicked(MouseEvent e) {

        
        if(e.getSource() instanceof Splash){
            this.getLayeredPane().remove(splash);
            simulator.setSelected(true);
            sum.setSelected(true);
            this.update();
            
        } else {
        
            Cell obj = (Cell)e.getSource();

            if(tip == null && currentCursorButton == helpButton && currentModality == AbacoFrame.EXE && obj.isCliccable(AbacoFrame.EXE)){
                Point point = new Point(e.getXOnScreen() - this.getX(), e.getYOnScreen()-this.getY()-10);
                tip = new Tip(this.getCorrectValue(obj), point);
                this.getLayeredPane().add(tip, 0);
                this.update();
            }
        }
    }

    /**
     * Ascoltatore per gestire la pressione del mouse all'interno della cella.
     * In questo metodo vienne preso il tip e portato di fronte a tutti gli altri
     * oggetti grafici.
     * Non sarebbe necessario aggiungere questa parte di codice ma viene rilevato
     * un bug grafico se viene ricliccata la cella, pertanto è necessario per dare
     * all'utente un'esperienza grafica senza fastidi di sorta.
     */
    @Override
    public void mousePressed(MouseEvent e) {
        if(tip != null){
            this.getLayeredPane().moveToFront(tip);
            this.update();
        }
    }

    /**
     * Ascoltatore non utilizzato.
     */
    @Override
    public void mouseReleased(MouseEvent e) {
    }

    /**
     * Ascoltatore non utilizzato.
     */
    @Override
    public void mouseEntered(MouseEvent e) {
    }

    
    /**
     * Ascoltatore per gestire l'uscita del mouse dalla cella.
     * In questo metodo viene distrutto il tip e quindi non viene più visualizzato.
     */
    @Override
    public void mouseExited(MouseEvent e) {
        if(tip != null){
            this.getLayeredPane().remove(tip);
            tip = null;
            this.update();
        }
    }

    /**
     * Ascoltatore non utilizzato.
     */
    @Override
    public void mouseDragged(MouseEvent e) {
    }

    /**
     * Ascoltatore per gestire il movimento del mouse all'interno della cella.
     * In questo metodo viene spostato il tip per mantenerlo vicino il cursore.
     */
    @Override
    public void mouseMoved(MouseEvent e) {
        if(tip != null)
            tip.setLocation(e.getXOnScreen() - this.getX(), e.getYOnScreen()-this.getY()-10);
    }
}