package abacomayadecimale;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import javax.swing.BorderFactory;
import javax.swing.JPanel;

/**
 * Classe astratta per la gestione delle celle nell'abaco.
 * Questa classe viene estesa da SCell e DCell.
 * @see SCell
 * @see DCell
 */
public abstract class Cell extends JPanel {
    
    /**
     * Dimensione della cella.
     */
    protected static final int DIM = 70;
    
    /**
     * Font utilizzato per la stampa delle coordinate della cella.
     */
    public final static Font FONT = new Font(Font.SERIF, Font.PLAIN, 10);
    
    /**
     * Numero di riga.
     */
    protected int row;
    
    /**
     * Numeoro di colonna.
     */
    protected int col;
    
    /**
     * Costruttore di Cell.
     * Viene creata una cella astratta con una coordinata (<code>row</code>|<code>col</code>)
     * per facilitare l'utente alla comprensione degli step in modalità simulazione 
     * e alla correzione in modalità esercitazione. 
     * 
     * <br>Per indicare solo la riga in cui si trova la cella, basta inserire il valore
     * -1 nel parametro <code>col</code>, viceversa inserire -1 nel parametro <code>row</code>.
     * 
     * @param type tipo della cella
     * @param row riga in cui si trova la cella
     */
    Cell(int row, int col){
        this.row = row;
        this.col = col;
        
        //Imposta la dimensione
        this.setPreferredSize(new Dimension(DIM, DIM));
        this.setMaximumSize(new Dimension(DIM, DIM));
        //Imposta il bordo
        this.setBorder(BorderFactory.createLineBorder(Color.black));
    }
    
    /**
     * Aggiorna graficamente la cella.
     */
    protected void update(){
        this.revalidate();
        this.repaint();
    }
    
    /**
     * Ritorna il numero di riga in cui si trova la cella.
     * @return numero di riga
     */
    public int getRow(){
        return row;
    }
    
    
    /**
     * Ritorna il numero di colonna in cui si trova la cella.
     * @return numero di colonna
     */
    public int getCol(){        
        return col;
    }
    
    /**
     * Ritorna il valore decimale dell'unione degli elmenti dell'intera cella.
     * @return un intero con il valore decimale dell'intera cella
     */
    public abstract int getValue();
    
    /**
     * Svuota al cella.
     */
    public abstract void reset();
    
    /**
     * Imposta un valore nella cella.
     * Il valore <code>x</code> è in decimale quindi viene convertito e successivamente
     * vengono impostati i frijolito e i palito corrispondenti.
     * @param x valore da inserire nella cella
     */
    public abstract void setValue(int x);
    
    /**
     * Stabilisce se la cella è cliccabile dall'utente.
     * Se torna TRUE la cella è cliccabile dall'utente, viceversa l'utente non 
     * potrà cliccare sulla cella.
     * @param modality modalità in cui l'utente tenta di cliccare sulla cella
     * @return TRUE se la cella è cliccabile <br> FALSE se non è cliccabile
     */
    public abstract boolean isCliccable(int modality);
    
}
