package abacomayadecimale;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

/**
 * Classe per l'implementazione della cella singola nell'abaco.
 */
public class SCell extends Cell implements MouseListener {
    
    /**
     * Costante per determinare se la cella si trova in SMatrix.
     * @see Abaco#SMatrix
     */
    public static final int ABACO = 0;
    
    /**
     * Costante per determinare se la cella si trova in up.
     * @see Abaco#up
     */
    public static final int UP = 1;
    
    /**
     * Costante per determinare se la cella si trova in right.
     * @see Abaco#right
     */
    public static final int RIGHT = 2;
    
    /**
     * Costante per determinare se la cella si trova in down.
     * @see Abaco#down
     */
    public static final int DOWN = 3;
    
    /**
     * Costante per determinare se la cella si trova in left.
     * @see Abaco#left
     */
    public static final int LEFT = 4;

    /**
     * Larghezza caracol.
     */
    private final static int wCara = 60;
    
    /**
     * Altezza caracol.
     */
    private final static int hCara = 36;
    
    /**
     * Larghezza frijolito.
     */
    private final static int wFrij = 5;
    
    /**
     * Altezza frijolito.
     */
    private final static int hFrij = 5;
    
    /**
     * Larghezza palito.
     */
    private final static int wPali = 65;
    
    /**
     * Altezza palito.
     */
    private final static int hPali = 2;
    
    /**
     * Distanza tra i frijolito.
     */
    private final static int distFrij = 8;
    
    /**
     * Distanza tra un frijolito e un palito e/o tra i palito.
     */
    private final static int dist = 4;
    
    /**
     * Tipo di cella, ovvero posizionamento della cella nell'abaco.
     * @see SCell#ABACO
     * @see SCell#UP
     * @see SCell#RIGHT
     * @see SCell#DOWN
     * @see SCell#LEFT
     */
    private int type;
    
    /**
     * Boolean che permette la visualizzazione del Caracol all'interno della cella.
     * Il caracol ha valore 0.
     */
    private boolean caracol;
    
    /**
     * Numero di frijolito contenuti nella cella.
     * Il frijolito ha valore 1.
     */
    private int frijolito;
    
    /**
     * Numero di palito contenuti nella cella.
     * Il palito ha valore 5.
     */
    private int palito;

    /**
     * Boolean che permette alla cella di essere modificata dall'utente.
     * Se il valore è true l'utente può modificare la cella nel caso in cui sia
     * selezionata l'addizione come operazione e che l'abaco sia in modalità simulazione.
     */
    private boolean sumSim;
    
    /**
     * Boolean che permette alla cella di essere modificata dall'utente.
     * Se il valore è true l'utente può modificare la cella nel caso in cui sia
     * selezionata la sottrazione come operazione e che l'abaco sia in modalità simulazione.
     */
    private boolean subSim;
    
    /**
     * Boolean che permette alla cella di essere modificata dall'utente.
     * Se il valore è true l'utente può modificare la cella nel caso in cui sia
     * selezionata la moltiplicazione come operazione e che l'abaco sia in modalità simulazione.
     */
    private boolean mulSim;
    
    /**
     * Boolean che permette alla cella di essere modificata dall'utente.
     * Se il valore è true l'utente può modificare la cella nel caso in cui sia
     * selezionata la divisione come operazione e che l'abaco sia in modalità simulazione.
     */
    private boolean divSim;
    
    /**
     * Boolean che permette alla cella di essere modificata dall'utente.
     * Se il valore è true l'utente può modificare la cella nel caso in cui sia
     * selezionata l'addizione come operazione e che l'abaco sia in modalità esercitazione.
     */
    private boolean sumExe;
    
    /**
     * Boolean che permette alla cella di essere modificata dall'utente.
     * Se il valore è true l'utente può modificare la cella nel caso in cui sia
     * selezionata la sottrazione come operazione e che l'abaco sia in modalità esercitazione.
     */
    private boolean subExe;
    
    /**
     * Boolean che permette alla cella di essere modificata dall'utente.
     * Se il valore è true l'utente può modificare la cella nel caso in cui sia
     * selezionata la moltiplicazione come operazione e che l'abaco sia in modalità esercitazione.
     */
    private boolean mulExe;
    
    /**
     * Boolean che permette alla cella di essere modificata dall'utente.
     * Se il valore è true l'utente può modificare la cella nel caso in cui sia
     * selezionata la divisione come operazione e che l'abaco sia in modalità esercitazione.
     */
    private boolean divExe;
    
    /**
     * Costruttore della Single Cell (SCell).
     * Viene creata una cella singola, con una coordinata (<code>row</code>|<code>col</code>)
     * per facilitare l'utente alla comprensione degli step in modalità simulazione 
     * e alla correzione in modalità esercitazione. 
     * Si deve specificare tipo di cella, la posizione rispetto l'abaco, in quale tipo
     * di operazione e tipo di modalità la cella può essere modificata dall'utente.
     * 
     * <br>Per indicare solo la riga in cui si trova la cella, basta inserire il valore
     * -1 nel parametro <code>col</code>, viceversa inserire -1 nel parametro <code>row</code>.
     * 
     * @param type tipo della cella
     * @param row riga in cui si trova la cella
     * @param col colonna in cui si trova la cella
     * @param sumSim 
     * @param subSim
     * @param mulSim
     * @param divSim
     * @param sumExe
     * @param subExe
     * @param mulExe
     * @param divExe 
     */
    public SCell(int type, int row, int col, boolean sumSim, boolean subSim, boolean mulSim, boolean divSim, boolean sumExe, boolean subExe, boolean mulExe, boolean divExe){
        super(row, col);
        //Imposta i valori
        this.type = type;
        this.caracol = false;
        this.frijolito = 0;
        this.palito = 0;
        this.sumSim = sumSim;
        this.subSim = subSim;
        this.mulSim = mulSim;
        this.divSim = divSim;
        this.sumExe = sumExe;
        this.subExe = subExe;
        this.mulExe = mulExe;
        this.divExe = divExe;
        
        //Imposta l'ascoltatore del mouse
        this.addMouseListener(this);
            
        this.update();   
    }
    
        @Override
    public void paintComponent(Graphics g){
        super.paintComponent(g);
        
        //Imposta lo sfondo
        g.setColor(Color.CYAN);
        
        if(AbacoFrame.currentModality == AbacoFrame.SIM){
            switch(AbacoFrame.currentOperation){
                case Abaco.SUM:
                    if(sumSim)
                        g.fillRect(0, 0, DIM, DIM);
                    break;

                case Abaco.SUB:
                    if(subSim)
                        g.fillRect(0, 0, DIM, DIM);
                    break;

                case Abaco.MUL:
                    if(mulSim)
                        g.fillRect(0, 0, DIM, DIM);
                    break;

                case Abaco.DIV:
                    if(divSim)
                        g.fillRect(0, 0, DIM, DIM);
                    break;
            }
        } else {
            switch(AbacoFrame.currentOperation){
                case Abaco.SUM:
                    if(sumExe)
                        g.fillRect(0, 0, DIM, DIM);
                    break;

                case Abaco.SUB:
                    if(subExe)
                        g.fillRect(0, 0, DIM, DIM);
                    break;

                case Abaco.MUL:
                    if(mulExe)
                        g.fillRect(0, 0, DIM, DIM);
                    break;

                case Abaco.DIV:
                    if(divExe)
                        g.fillRect(0, 0, DIM, DIM);
                    break;
            }
        }
        
        //Scrive il numero di riga o/e colonna
        g.setColor(Color.BLACK);
        g.setFont(Cell.FONT);
        
        if(col == -1)
            g.drawString(""+(4-row), 2, 10);
        else if(row == -1)
            g.drawString(""+(4-col), 2, 10);
        else g.drawString((4-row)+"|"+(4-col), 2, 10);
        
        //Disegna il Caracol
        if(caracol){
            int xCaracol = (DIM-wCara)/2;
            int yCaracol = (DIM-hCara)/2;
            
            g.drawOval(xCaracol, yCaracol, wCara, hCara);           //Ovale esterno
            g.drawArc(xCaracol+12, yCaracol+3, 8, 30, 90, 180);     //Primo arco
            g.drawArc(xCaracol+26, yCaracol+0, 8, 35, 90, 180);     //Secondo arco
            g.drawArc(xCaracol+40, yCaracol+3, 8, 30, 90, 180);     //Terzo arco
            return;
        }
        
        int unities = frijolito%5;
        int rowsFive = frijolito/5;
        int rows = (unities>0? 1:0) + rowsFive;

        int x = (DIM -(wFrij*unities) -(distFrij*(unities-1)))/2;
        int y = (DIM -(hFrij*rows) -(dist*(rows-1)) -(palito>0? (dist +(hPali*palito) +(dist*(palito-1))):0))/2;
        
        //Disegna fino a 4 frijolito
        for(int i = 0; i<unities; i++){
            x += (i>0? (wFrij+distFrij):0);
            g.fillRect(x, y, wFrij, hFrij);    
        }
        
        //Disegna cinquine di frijolito
        x = (DIM -(wFrij*5) -(distFrij*4))/2;
        if(unities>0)
            y+= hFrij + dist;
        
        for(int j = 0; j<rowsFive; j++){
            for(int i = 0; i<5; i++){
                int xFives = x +(wFrij+distFrij)*i;
                g.fillRect(xFives, y, wFrij, hFrij);    
            }
            y += hFrij+dist;
        }

        //Disegna i palito
        x = (DIM - wPali)/2;
        for(int i = 0; i<palito; i++){
            g.fillRect(x, y, wPali, hPali);
            y += hPali+dist;
        }        
    }
    
    /**
     * Determina se è presente il caracol.
     * @return TRUE se il valore della cella = 0 quindi è presente il caracol <br> FALSE se il valore della cella > 0
     * oppure la cella è vuota
     */
    public boolean isCaracol(){
        return caracol;
    }

    /**
     * Ritorna il numero di frijolito contenuti nella cella.
     * @return numero di frijolito contenuti nella cella
     */
    public int getFrijolito(){
        return frijolito;
    }
    
    /**
     * Ritorna il numero di palito contenuti nella cella.
     * @return numero di palito contenuti nella cella
     */
    public int getPalito(){
        return palito;
    }

    @Override
    public int getValue(){
        return palito*5 + frijolito;
    }
    
    @Override
    public void reset(){
        caracol = false;
        frijolito = 0;
        palito = 0;
        this.update();
    }
    
    /**
     * Imposta il numero di frijolito nella cella.
     * @param frijolito valore con il numero di frijolito da inserire nella cella
     */
    public void setFrijolito(int frijolito){
        this.frijolito = frijolito;
        caracol = (this.getValue() == 0);
        this.update();
    }
    
    /**
     * Imposta il numero di palito nella cella.
     * @param palito valore con il numero di palito da inserire nella cella
     */
    public void setPalito(int palito){
        this.palito = palito;
        caracol = (this.getValue() == 0);
        this.update();
    }

    @Override
    public void setValue(int x){
        frijolito = x%5;
        palito = x/5;
        caracol = (this.getValue() == 0);
        this.update();
    }
    
    /**
     * Aggiunge un valore alla cella.
     * Il valore <code>x</code> è in decimale quindi viene convertito e successivamente
     * vengono aggiunti i frijolito e i palito corrispondenti a quelli esistenti.
     */
    public void addValue(int x){
        
        frijolito += x%5;
        palito += x/5;
        
        palito = palito + frijolito/5;
        frijolito = frijolito%5;
        
        caracol = (this.getValue() == 0);

        this.update();
    }

    /**
     * Ritorna il tipo di cella, ovvero la posizione della cella nll'abaco.
     * @return il tipo di cella.
     * @see SCell#ABACO
     * @see SCell#UP
     * @see SCell#RIGHT
     * @see SCell#DOWN
     * @see SCell#LEFT
     */
    public int getType(){
        return type;
    }
    
    /**
     * Gestisce il clic da parte dell'utente con i vari cursori quando l'abaco 
     * è modalità simulazione.
     */
    private void clickSim(){
        
        int value = this.getValue();
        
        switch(this.getCursor().getName()){
            
            case "cross":
                caracol = false;
                frijolito = 0;
                palito = 0;
                break;

            case "caracol":
                caracol = true;
                frijolito = 0;
                palito = 0;
                break;

            case "frijolito":
                if(value+1 < 10)
                    this.addValue(1);
                break;

            case "palito":
                if(value+5 < 10)
                    this.addValue(5);
                break;

            case "crossMinus":
                caracol = false;
                frijolito = 0;
                palito = 0;
                break;
                    
            case "caracolMinus":
                if(caracol == true)
                    caracol = false;
                break;

            case "frijolitoMinus":
                if(value-1 >= 0)
                    this.setValue(this.getValue()-1);
                break;

            case "palitoMinus":
                if(palito > 0)
                    palito--;
                break;
        }
        
        this.update();
    }
    
    /**
     * Gestisce il clic da parte dell'utente con i vari cursori quando l'abaco 
     * è modalità simulazione.
     */
    private void clickExe(){
                
        switch(this.getCursor().getName()){
            case "cross":
                caracol = false;
                frijolito = 0;
                palito = 0;
                break;

            case "caracol":
                caracol = true;
                frijolito = 0;
                palito = 0;
                break;

            case "frijolito":
                frijolito++;
                break;

            case "palito":
                palito++;
                break;
                
            case "crossMinus":
                caracol = false;
                frijolito = 0;
                palito = 0;
                break;

            case "caracolMinus":
                if(caracol == true)
                    caracol = false;
                break;

            case "frijolitoMinus":
                if(frijolito>0)
                    frijolito--;
                break;

            case "palitoMinus":
                if(palito>0)
                    palito--;
                break;
        }
        
        this.update();
    }

    @Override
    public boolean isCliccable(int modality){
                
        if(modality == AbacoFrame.SIM){
            switch(AbacoFrame.currentOperation){
                case Abaco.SUM:
                    return sumSim;

                case Abaco.SUB:
                    return subSim;

                case Abaco.MUL:
                    return mulSim;

                case Abaco.DIV:
                    return divSim;
            }
        } else {
            switch(AbacoFrame.currentOperation){
                case Abaco.SUM:
                    return sumExe;

                case Abaco.SUB:
                    return subExe;

                case Abaco.MUL:
                    return mulExe;

                case Abaco.DIV:
                    return divExe;
            }
        }
        
        return false;
    }

    @Override
    public void mousePressed(MouseEvent e) {
        if(this.isCliccable(AbacoFrame.currentModality)){
            if(AbacoFrame.currentModality == AbacoFrame.SIM)
                clickSim();
            else clickExe();
        }
    }

    @Override
    public void mouseClicked(MouseEvent e) {
    }

    @Override
    public void mouseReleased(MouseEvent e) {
    }

    @Override
    public void mouseEntered(MouseEvent e) {
    }

    @Override
    public void mouseExited(MouseEvent e) {
    }
}